from django.core.management import call_command
from django.conf import settings
from translate_projects_django.manage_translations import generate_translations
import os
from translate_projects_django.api_utils import Api

def make_translations(self,locale_param):    

    languages = getattr(settings, 'LANGUAGES', [])
    locale_dir = getattr(settings, 'LOCALE_PATHS', [])[0]
    default_language = getattr(settings, 'LANGUAGE_CODE', '')

    self.api_client = Api()
    # Generate translations for the default language ('es' in this case)
    generate_translations(self, 'es', locale_dir)
    
    if locale_param:
        # Check if the provided locale is valid
        if locale_param not in dict(languages):
            self.stdout.write(self.style.ERROR(f"The language '{locale_param}' is not valid."))
            self.stdout.write(self.style.ERROR(f"Available languages: {', '.join([lang[0] for lang in languages])}"))
            self.stdout.write(self.style.ERROR(f"You can add the language in the settings.py file under LANGUAGES=[...]"))
            return

        # Warn if the locale is the same as the default language
        if locale_param == default_language:
            self.stdout.write(self.style.WARNING(f"The language '{locale_param}' is not translated, as the default language '{default_language}' has already been translated."))
            return

        # Translate for the specific locale
        self.stdout.write(self.style.SUCCESS(f"\n🚀 Translating for language: {locale_param}"))
        generate_translations(self, locale_param, locale_dir)
        self.stdout.write(self.style.SUCCESS(f"\n🎉 Translation generation for '{locale_param}' completed successfully ✅\n"))

        # Compile messages
        self.stdout.write(self.style.WARNING(f"\n🛠 Compiling messages..."))
        call_command('compilemessages',verbosity=0)
        return

    # If no specific locale is passed, generate translations for all languages except the default language
    for code, name in languages:
        if code != default_language:
            generate_translations(self, code, locale_dir)

    # Compile messages after translations are generated for all languages
    self.stdout.write(self.style.WARNING(f"\n🛠 Compiling messages for all languages..."))
    call_command('compilemessages',verbosity=0)
    self.stdout.write(self.style.SUCCESS(f"\n🎉 Translation generation for all languages completed successfully ✅\n"))


from django.core.management.base import BaseCommand
from django.conf import settings
import subprocess
from django.core.management import call_command
import polib
import json
import os
import requests
from translate_projects_django.cases.make_translations import make_translations

class Command(BaseCommand):
    help = "Translate the project for a specific language or all languages."

    def add_arguments(self, parser):
        # Add an optional argument for the locale
        parser.add_argument(
            '--locale', 
            type=str, 
            help='Specific language to translate, e.g., "ko" for Korean.'
        )
        # get translations api
        parser.add_argument(
            '--get',
            action='store_true',
            help='Get translations from API',
            default=False
        )

    def handle(self, *args, **kwargs):
        # Get the locale argument
        locale_param = kwargs.get('locale')

        # Get the get argument
        get_translations = kwargs.get('get')

        if get_translations:
            #logic for get translations
            return

        if not get_translations:
            make_translations(self, locale_param)
            return


import sys
from django.core.management.base import BaseCommand

class StyledMessages(BaseCommand):
    def __init__(self):
        super().__init__()  

    def error(self, message):
        sys.stderr.write(self.style.ERROR(f"{message}\n"))

    def success(self, message):
        sys.stdout.write(self.style.SUCCESS(f"{message}\n"))

    def warning(self, message):
        sys.stdout.write(self.style.WARNING(f"{message}\n")) 



import sys
import requests
from django.conf import settings
from .style import StyledMessages

def get_error_message(response):
    """Extract error message from API response."""
    try:
        return response.json().get('message', 'Unknown error')
    except ValueError:
        return 'Error on updating translation data'


API_BASE_URL = "https://api.translateprojects.dev"

styled_messages = StyledMessages()

class Api:
    def __init__(self, base_url=API_BASE_URL, token=None):
        self.base_url = base_url.rstrip('/')
        self.token = token or getattr(settings, "TRANSLATE_PROJECTS_API_KEY", None)
        self.headers = {
            'Content-Type': 'application/json',
            'Accept': 'application/json',
        }
        if self.token:
            self.headers['Authorization'] = f"Token {self.token}"

    def make_request(self, method, endpoint, params=None, data=None, retries=3):
        """General method to handle API requests with retries."""
        url = f"{self.base_url}{endpoint}"
        for attempt in range(retries):
            try:
                response = requests.request(method, url, json=data, params=params, headers=self.headers)
                if response.status_code == 200:
                    return response.json()
                elif response.status_code in {524, 504}:  # Timeout errors
                    continue  # Retry if needed
                else:
                    error_message = get_error_message(response)
                    styled_messages.error(f"API Error ({response.status_code}): {error_message}")
                    sys.exit(1)
            except requests.RequestException as e:
                styled_messages.error(f"Request failed: {e}")
                sys.exit(1)
        
        styled_messages.error("Max retries reached. Request failed.")
        sys.exit(1)

    def get_translations(self, data, source_lang, target_lang, route_file=None):
        """Fetch translations from the API."""
        params = {
            'source_lang': source_lang,
            'target_lang': target_lang,
            'type_project': 'django'
        }
        if route_file:
            params['route_file'] = route_file
        
        response = self.make_request('POST', '/v1/translations/', params=params, data=data)
        return response.get('data', None)

    def fetch_translations(self, params=None):
        """Retrieve existing translations from the API."""
        params = params or {}
        response = self.make_request('GET', '/v1/translations/list', params=params)
        return response
